//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2007-2012, Image Engine Design Inc. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the name of Image Engine Design nor the names of any
//       other contributors to this software may be used to endorse or
//       promote products derived from this software without specific prior
//       written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
//  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef IECOREGL_NUMERICTRAITS_H
#define IECOREGL_NUMERICTRAITS_H

#include "IECoreGL/GL.h"

#include "IECore/Data.h"
#include "IECore/Export.h"

IECORE_PUSH_DEFAULT_VISIBILITY
#include "OpenEXR/half.h"
IECORE_POP_DEFAULT_VISIBILITY

namespace IECoreGL
{

/// This class provides information handy for writing template code
/// to operate with OpenGL.
template<typename T>
struct NumericTraits
{
	/// Returns the appropriate enum value to tell OpenGL the type
	/// of data being passed to it.
	static GLenum glType() { return 0; }
};

template<> IECORE_EXPORT
GLenum NumericTraits<GLbyte>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLubyte>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLshort>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLushort>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLint>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLuint>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<half>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLfloat>::glType();

template<> IECORE_EXPORT
GLenum NumericTraits<GLdouble>::glType();

IECOREGL_API GLenum glType( const IECore::Data *data );

} // namespace IECoreGL

#endif // IECOREGL_NUMERICTRAITS_H
